﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Windows.Markup;
using UIT.Converters;
using System.Globalization;
using UIT.Utils;

namespace UIT._2D
{
    /// <summary>Implements a structure that is used to describe the <see cref="T:System.Windows.Size" /> of an object. </summary>
#if !SILVERLIGHT
	[ValueSerializer(typeof(SizeValueSerializer))]
    [Serializable]
#endif
    [TypeConverter(typeof(SizeConverter))]
    public struct Size : IFormattable
    {
        internal double _width;
        internal double _height;
        private static readonly Size s_empty = Size.CreateEmptySize();
        /// <summary>Gets a value that represents a static empty <see cref="T:System.Windows.Size" />. </summary>
        /// <returns>An empty instance of <see cref="T:System.Windows.Size" />.</returns>
        public static Size Empty
        {
            get
            {
                return Size.s_empty;
            }
        }
        /// <summary>Gets a value that indicates whether this instance of <see cref="T:System.Windows.Size" /> is <see cref="P:System.Windows.Size.Empty" />. </summary>
        /// <returns>true if this instance of size is <see cref="P:System.Windows.Size.Empty" />; otherwise false.</returns>
        public bool IsEmpty
        {
            get
            {
                return this._width < 0.0;
            }
        }
        /// <summary>Gets or sets the <see cref="P:System.Windows.Size.Width" /> of this instance of <see cref="T:System.Windows.Size" />. </summary>
        /// <returns>The <see cref="P:System.Windows.Size.Width" /> of this instance of <see cref="T:System.Windows.Size" />. The default value is 0. The value cannot be negative.</returns>
        public double Width
        {
            get
            {
                return this._width;
            }
            set
            {
                if (this.IsEmpty)
                {
                    throw new InvalidOperationException("CannotModifyEmptySize");
                }
                if (value < 0.0)
                {
                    throw new ArgumentException("WidthCannotBeNegative");
                }
                this._width = value;
            }
        }
        /// <summary>Gets or sets the <see cref="P:System.Windows.Size.Height" /> of this instance of <see cref="T:System.Windows.Size" />. </summary>
        /// <returns>The <see cref="P:System.Windows.Size.Height" /> of this instance of <see cref="T:System.Windows.Size" />. The default is 0. The value cannot be negative.</returns>
        public double Height
        {
            get
            {
                return this._height;
            }
            set
            {
                if (this.IsEmpty)
                {
                    throw new InvalidOperationException("CannotModifyEmptySize");
                }
                if (value < 0.0)
                {
                    throw new ArgumentException("HeightCannotBeNegative");
                }
                this._height = value;
            }
        }
        /// <summary>Compares two instances of <see cref="T:System.Windows.Size" /> for equality. </summary>
        /// <returns>true if the two instances of <see cref="T:System.Windows.Size" /> are equal; otherwise false.</returns>
        /// <param name="size1">The first instance of <see cref="T:System.Windows.Size" /> to compare.</param>
        /// <param name="size2">The second instance of <see cref="T:System.Windows.Size" /> to compare.</param>
        public static bool operator ==(Size size1, Size size2)
        {
            return size1.Width == size2.Width && size1.Height == size2.Height;
        }
        /// <summary>Compares two instances of <see cref="T:System.Windows.Size" /> for inequality. </summary>
        /// <returns>true if the instances of <see cref="T:System.Windows.Size" /> are not equal; otherwise false.</returns>
        /// <param name="size1">The first instance of <see cref="T:System.Windows.Size" /> to compare.</param>
        /// <param name="size2">The second instance of <see cref="T:System.Windows.Size" /> to compare.</param>
        public static bool operator !=(Size size1, Size size2)
        {
            return !(size1 == size2);
        }
        /// <summary>Compares two instances of <see cref="T:System.Windows.Size" /> for equality. </summary>
        /// <returns>true if the instances of <see cref="T:System.Windows.Size" /> are equal; otherwise, false.</returns>
        /// <param name="size1">The first instance of <see cref="T:System.Windows.Size" /> to compare.</param>
        /// <param name="size2">The second instance of <see cref="T:System.Windows.Size" /> to compare.</param>
        public static bool Equals(Size size1, Size size2)
        {
            if (size1.IsEmpty)
            {
                return size2.IsEmpty;
            }
            return size1.Width.Equals(size2.Width) && size1.Height.Equals(size2.Height);
        }
        /// <summary>Compares an object to an instance of <see cref="T:System.Windows.Size" /> for equality. </summary>
        /// <returns>true if the sizes are equal; otherwise, false.</returns>
        /// <param name="o">The <see cref="T:System.Object" /> to compare.</param>
        public override bool Equals(object o)
        {
            if (o == null || !(o is Size))
            {
                return false;
            }
            Size size = (Size)o;
            return Size.Equals(this, size);
        }
        /// <summary>Compares a value to an instance of <see cref="T:System.Windows.Size" /> for equality. </summary>
        /// <returns>true if the instances of <see cref="T:System.Windows.Size" /> are equal; otherwise, false.</returns>
        /// <param name="value">The size to compare to this current instance of <see cref="T:System.Windows.Size" />.</param>
        public bool Equals(Size value)
        {
            return Size.Equals(this, value);
        }
        /// <summary>Gets the hash code for this instance of <see cref="T:System.Windows.Size" />. </summary>
        /// <returns>The hash code for this instance of <see cref="T:System.Windows.Size" />.</returns>
        public override int GetHashCode()
        {
            if (this.IsEmpty)
            {
                return 0;
            }
            return this.Width.GetHashCode() ^ this.Height.GetHashCode();
        }
        /// <summary>Returns an instance of <see cref="T:System.Windows.Size" /> from a converted <see cref="T:System.String" />. </summary>
        /// <returns>An instance of <see cref="T:System.Windows.Size" />.</returns>
        /// <param name="source">A <see cref="T:System.String" /> value to parse to a <see cref="T:System.Windows.Size" /> value.</param>
        public static Size Parse(string source)
        {
            TokenizerHelper tokenizerHelper = new TokenizerHelper(source, CultureInfo.InvariantCulture);
            string text = tokenizerHelper.NextTokenRequired();
            Size empty;
            if (text == "Empty")
            {
                empty = Size.Empty;
            }
            else
            {
                empty = new Size(Convert.ToDouble(text, CultureInfo.InvariantCulture), Convert.ToDouble(tokenizerHelper.NextTokenRequired(), CultureInfo.InvariantCulture));
            }
            tokenizerHelper.LastTokenRequired();
            return empty;
        }
        /// <summary>Returns a <see cref="T:System.String" /> that represents this <see cref="T:System.Windows.Size" /> object. </summary>
        /// <returns>A <see cref="T:System.String" /> that represents this instance of <see cref="T:System.Windows.Size" />.</returns>
        public override string ToString()
        {
            return this.ConvertToString(null, null);
        }
        /// <summary>Returns a <see cref="T:System.String" /> that represents this instance of <see cref="T:System.Windows.Size" />. </summary>
        /// <returns>A <see cref="T:System.String" /> that represents this <see cref="T:System.Windows.Size" /> object.</returns>
        /// <param name="provider">An object that provides a way to control formatting.</param>
        public string ToString(IFormatProvider provider)
        {
            return this.ConvertToString(null, provider);
        }
        string IFormattable.ToString(string format, IFormatProvider provider)
        {
            return this.ConvertToString(format, provider);
        }
        internal string ConvertToString(string format, IFormatProvider provider)
        {
            if (this.IsEmpty)
            {
                return "Empty";
            }
            char numericListSeparator = TokenizerHelper.GetNumericListSeparator(provider);
            return string.Format(provider, string.Concat(new string[]
			{
				"{1:", 
				format, 
				"}{0}{2:", 
				format, 
				"}"
			}), new object[]
			{
				numericListSeparator, 
				this._width, 
				this._height
			});
        }
        /// <summary>Initializes a new instance of the <see cref="T:System.Windows.Size" /> structure and assigns it an initial <paramref name="width" /> and <paramref name="height" />.</summary>
        /// <param name="width">The initial width of the instance of <see cref="T:System.Windows.Size" />.</param>
        /// <param name="height">The initial height of the instance of <see cref="T:System.Windows.Size" />.</param>
        public Size(double width, double height)
        {
            if (width < 0.0 || height < 0.0)
            {
                throw new ArgumentException("WidthAndHeightCannotBeNegative");
            }
            this._width = width;
            this._height = height;
        }
        public static explicit operator Vector(Size size)
        {
            return new Vector(size._width, size._height);
        }
        public static explicit operator Point(Size size)
        {
            return new Point(size._width, size._height);
        }
        private static Size CreateEmptySize()
        {
            return new Size
            {
                _width = double.NegativeInfinity,
                _height = double.NegativeInfinity
            };
        }
    }
}
