﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Globalization;
using UIT._2D;
using System.Windows.Markup;

namespace UIT.Converters
{
    /// <summary>Converts instances of other types to and from a <see cref="T:System.Windows.Point" />. </summary>
    public sealed class PointConverter : TypeConverter
    {
        /// <summary>Determines whether an object can be converted from a given type to an instance of a <see cref="T:System.Windows.Point" />.  </summary>
        /// <returns>true if the type can be converted to a <see cref="T:System.Windows.Point" />; otherwise, false.</returns>
        /// <param name="context">Describes the context information of a type.</param>
        /// <param name="sourceType">The type of the source that is being evaluated for conversion.</param>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return sourceType == typeof(string) || base.CanConvertFrom(context, sourceType);
        }
        /// <summary>Determines whether an instance of a <see cref="T:System.Windows.Point" /> can be converted to a different type. </summary>
        /// <returns>true if this <see cref="T:System.Windows.Point" /> can be converted to <paramref name="destinationType" />; otherwise, false.</returns>
        /// <param name="context">Describes the context information of a type.</param>
        /// <param name="destinationType">The desired type this <see cref="T:System.Windows.Point" /> is being evaluated for conversion.</param>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return destinationType == typeof(string) || base.CanConvertTo(context, destinationType);
        }
        /// <summary>Attempts to convert the specified object to a <see cref="T:System.Windows.Point" />. </summary>
        /// <returns>The <see cref="T:System.Windows.Point" /> created from converting <paramref name="value" />.</returns>
        /// <param name="context">Provides contextual information required for conversion.</param>
        /// <param name="culture">Cultural information to respect during conversion.</param>
        /// <param name="value">The object being converted.</param>
        /// <exception cref="T:System.NotSupportedException">Thrown if the specified object is NULL or is a type that cannot be converted to a <see cref="T:System.Windows.Point" />.</exception>
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
#if !SILVERLIGHT
            if (value == null)
            {
                throw base.GetConvertFromException(value);
            }
#endif
            string text = value as string;
            if (text != null)
            {
                return Point.Parse(text);
            }
            return base.ConvertFrom(context, culture, value);
        }
        /// <summary>Attempts to convert a <see cref="T:System.Windows.Point" /> to a specified type. </summary>
        /// <returns>The object created from converting this <see cref="T:System.Windows.Point" />.</returns>
        /// <param name="context">Provides contextual information required for conversion.</param>
        /// <param name="culture">Cultural information to respect during conversion.</param>
        /// <param name="value">The <see cref="T:System.Windows.Point" /> to convert.</param>
        /// <param name="destinationType">The type to convert this <see cref="T:System.Windows.Point" /> to.</param>
        /// <exception cref="T:System.NotSupportedException">Thrown if <paramref name="value" /> is null or is not a <see cref="T:System.Windows.Point" />, or if the <paramref name="destinationType" /> is not one of the valid types for conversion.</exception>
        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType != null && value is Point)
            {
                Point point = (Point)value;
                if (destinationType == typeof(string))
                {
                    return point.ConvertToString(null, culture);
                }
            }
            return base.ConvertTo(context, culture, value, destinationType);
        }
        /// <summary>Initializes a new instance of the <see cref="T:System.Windows.PointConverter" /> class. </summary>
        public PointConverter()
        {
        }
    }

    /// <summary>Converts instances of <see cref="T:System.String" /> to and from instances of <see cref="T:System.Windows.Point" />.</summary>
#if !SILVERLIGHT
    public class PointValueSerializer : ValueSerializer
    {
        /// <summary>Determines whether the specified <see cref="T:System.String" /> can be converted to an instance of <see cref="T:System.Windows.Point" />.</summary>
        /// <returns>Always returns true.</returns>
        /// <param name="value">String to evaluate for conversion.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override bool CanConvertFromString(string value, IValueSerializerContext context)
        {
            return true;
        }
        /// <summary>Determines whether the specified <see cref="T:System.Windows.Point" /> can be converted to a <see cref="T:System.String" />.</summary>
        /// <returns>true if <paramref name="value" /> can be converted into a <see cref="T:System.String" />; otherwise, false.</returns>
        /// <param name="value">The object to evaluate for conversion.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override bool CanConvertToString(object value, IValueSerializerContext context)
        {
            return value is Point;
        }
        /// <summary>Converts a <see cref="T:System.String" /> into a <see cref="T:System.Windows.Point" />.</summary>
        /// <returns>A new instance of <see cref="T:System.Windows.Point" /> based on the supplied <paramref name="value" />.</returns>
        /// <param name="value">The string to convert.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override object ConvertFromString(string value, IValueSerializerContext context)
        {
            if (value != null)
            {
                return Point.Parse(value);
            }
            return base.ConvertFromString(value, context);
        }
        /// <summary>Converts an instance of <see cref="T:System.Windows.Point" /> to a <see cref="T:System.String" />.</summary>
        /// <returns>A string representation of the specified <see cref="T:System.Windows.Point" />.</returns>
        /// <param name="value">The object to convert into a string.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override string ConvertToString(object value, IValueSerializerContext context)
        {
            if (value is Point)
            {
                return ((Point)value).ConvertToString(null, CultureInfo.InvariantCulture);
            }
            return base.ConvertToString(value, context);
        }
        /// <summary>Initializes a new instance of the <see cref="T:System.Windows.Converters.PointValueSerializer" /> class.</summary>
        public PointValueSerializer()
        {
        }
    }
#endif
}
