﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Globalization;
using UIT._2D;
using System.Windows.Markup;

namespace UIT.Converters
{
    /// <summary> Converts instances of other types to and from a <see cref="T:System.Windows.Vector" />. </summary>
    public sealed class VectorConverter : TypeConverter
    {
        /// <summary>Indicates whether an object can be converted from a given type to an instance of a <see cref="T:System.Windows.Vector" />.</summary>
        /// <returns>true if objects of the specified type can be converted to a <see cref="T:System.Windows.Vector" />; otherwise, false.</returns>
        /// <param name="context">Describes the context information of a type.</param>
        /// <param name="sourceType">The source <see cref="T:System.Type" /> that is being queried for conversion support.</param>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            return sourceType == typeof(string) || base.CanConvertFrom(context, sourceType);
        }
        /// <summary>Determines whether instances of <see cref="T:System.Windows.Vector" /> can be converted to the specified type. </summary>
        /// <returns>true if instances of <see cref="T:System.Windows.Vector" /> can be converted to <paramref name="destinationType" />; otherwise, false.</returns>
        /// <param name="context">Describes the context information of a type.</param>
        /// <param name="destinationType">The desired type this <see cref="T:System.Windows.Vector" /> is being evaluated for conversion.</param>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            return destinationType == typeof(string) || base.CanConvertTo(context, destinationType);
        }
        /// <summary>Converts the specified object to a <see cref="T:System.Windows.Vector" />. </summary>
        /// <returns>The <see cref="T:System.Windows.Vector" /> created from converting <paramref name="value" />.</returns>
        /// <param name="context">Describes the context information of a type.</param>
        /// <param name="culture">Describes the <see cref="T:System.Globalization.CultureInfo" /> of the type being converted.</param>
        /// <param name="value">The object being converted.</param>
        public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
        {
#if !SILVERLIGHT
            if (value == null)
            {
                throw base.GetConvertFromException(value);
            }
#endif
            string text = value as string;
            if (text != null)
            {
                return Vector.Parse(text);
            }
            return base.ConvertFrom(context, culture, value);
        }
        /// <summary> Converts the specified <see cref="T:System.Windows.Vector" /> to the specified type. </summary>
        /// <returns>The object created from converting this <see cref="T:System.Windows.Vector" />.</returns>
        /// <param name="context">Describes the context information of a type.</param>
        /// <param name="culture">Describes the <see cref="T:System.Globalization.CultureInfo" /> of the type being converted.</param>
        /// <param name="value">The <see cref="T:System.Windows.Vector" /> to convert.</param>
        /// <param name="destinationType">The type to convert this <see cref="T:System.Windows.Vector" /> to.</param>
        public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
        {
            if (destinationType != null && value is Vector)
            {
                Vector vector = (Vector)value;
                if (destinationType == typeof(string))
                {
                    return vector.ConvertToString(null, culture);
                }
            }
            return base.ConvertTo(context, culture, value, destinationType);
        }
        /// <summary>Initializes a new instance of the <see cref="T:System.Windows.Vector" /> structure. </summary>
        public VectorConverter()
        {
        }
    }

#if !SILVERLIGHT
    /// <summary>Converts instances of <see cref="T:System.String" /> to and from instances of <see cref="T:System.Windows.Vector" />.</summary>
    public class VectorValueSerializer : ValueSerializer
    {
        /// <summary>Determines whether the specified <see cref="T:System.String" /> can be converted to an instance of <see cref="T:System.Windows.Vector" />.</summary>
        /// <returns>Always returns true.</returns>
        /// <param name="value">String to evaluate for conversion.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override bool CanConvertFromString(string value, IValueSerializerContext context)
        {
            return true;
        }
        /// <summary>Determines whether the specified <see cref="T:System.Windows.Vector" /> can be converted to a <see cref="T:System.String" />.</summary>
        /// <returns>true if <paramref name="value" /> can be converted into a <see cref="T:System.String" />; otherwise, false.</returns>
        /// <param name="value">The object to evaluate for conversion.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override bool CanConvertToString(object value, IValueSerializerContext context)
        {
            return value is Vector;
        }
        /// <summary>Converts a <see cref="T:System.String" /> into a <see cref="T:System.Windows.Vector" />.</summary>
        /// <returns>A new instance of <see cref="T:System.Windows.Vector" /> based on the supplied <paramref name="value" />.</returns>
        /// <param name="value">The string to convert.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override object ConvertFromString(string value, IValueSerializerContext context)
        {
            if (value != null)
            {
                return Vector.Parse(value);
            }
            return base.ConvertFromString(value, context);
        }
        /// <summary>Converts an instance of <see cref="T:System.Windows.Vector" /> to a <see cref="T:System.String" />.</summary>
        /// <returns>A string representation of the specified <see cref="T:System.Windows.Vector" />.</returns>
        /// <param name="value">The object to convert into a string.</param>
        /// <param name="context">Context information that is used for conversion.</param>
        public override string ConvertToString(object value, IValueSerializerContext context)
        {
            if (value is Vector)
            {
                return ((Vector)value).ConvertToString(null, CultureInfo.InvariantCulture);
            }
            return base.ConvertToString(value, context);
        }
        /// <summary>Initializes a new instance of the <see cref="T:System.Windows.Converters.VectorValueSerializer" /> class.</summary>
        public VectorValueSerializer()
        {
        }
    }
#endif
}
